<?php

App::uses('AppComponent', 'Controller/Component');

class FormsComponent extends AppComponent {

/**
 * Run the form injection. For custom fields this add type / validation arrays to the template for fields
 *
 * @param array    $data  Data from $this->request->data.
 * @param obj      $Model Model Object for the primary model we are dealing with.
 * @param int|null $id    The ID if on edit or null of add.
 * @return void.
 */
	public function injectAdminForm($data, $Model, $id) {
		$Model = $this->_controller->{$this->_controller->modelClass};

		if (isset($this->_controller->viewVars['fields'])) {
			$formModelAlias = $Model->Form->alias;

			$chosenOptions = [];
			if (empty(Configure::read('EvFormBuilder.associationsHaveMultipleForms'))) {
				$chosenOptions['max_selected_options'] = 1;
			}

			$formFields = [
				$formModelAlias . '.' . $formModelAlias => [
					'type' => 'chosen_multiselect',
					'label' => 'Form',
					'data-placeholder' => 'Select forms',
					'chosen_options' => $chosenOptions,
				]
			];

			$fields = $this->_controller->viewVars['fields'];

			$addBefore = 'is_active';
			if (!$Model->hasField('is_active')) {
				if ($Model->hasField('created')) {
					$addBefore = 'created';
				} else {
					$addBefore = null;
				}
			}

			if (!empty($addBefore)) {
				$fields = ArrayUtil::addBefore($fields, $Model->alias . '.is_active', $formFields);
			} else {
				$fields = Hash::merge($fields, $formFields);
			}

			$this->_controller->viewVars['fields'] = $fields;
			$this->_controller->set('forms', $Model->Form->getForDropDown());
		}
	}

/**
 * Add the provided form associations to the responses.
 *
 * @param array $responses    The responses that have been submitted.
 * @param array $associations The associations in [model => model_id] format.
 * @return array The responses with associations added.
 */
	public function addFormResponseAssociations($responses, $associations) {
		if (empty($this->_controller->Form)) {
			$this->_controller->loadModel('EvFormBuilder.Form');
		}

		return $this->_controller->Form->FormResponse->addFormResponseAssociations($responses, $associations);
	}

/**
 * Format the form responses so that they can be saved. Include the user who has submitted the response.
 *
 * @param array $responses The responses that have been submitted.
 * @return array The formatted responses.
 */
	public function formatFormResponseData($responses) {
		if (empty($this->_controller->Form)) {
			$this->_controller->loadModel('EvFormBuilder.Form');
		}

		foreach ($responses as &$response) {
			$response['user_id'] = $this->_getUserIdForResponse();

			$response = $this->_controller->Form->FormResponse->formatResponseData($response);
		}

		return $responses;
	}

/**
 * Get the id of the current user so that they can be associated with the response they have made.
 *
 * @return int The id of the user submitting the form response.
 */
	protected function _getUserIdForResponse() {
		return $this->_controller->Auth->user('User.id');
	}

/**
 * A user has responded to a form and it has saved successfully, redirect them to a success page and provide
 * a flash success message.
 *
 * @param int   $formId       The id of the form submitted.
 * @param array $requestQuery The query data from the request.
 * @return void.
 */
	public function responseSuccess($formId, $requestQuery) {
		$this->_controller->Flash->success('Your response has been saved');

		$redirect = [
			'action' => 'submit_success',
		];

		if (!empty(Configure::read('EvFormBuilder.redirects.responseSuccess'))) {
			$redirect = Configure::read('EvFormBuilder.redirects.responseSuccess');
		}

		$this->_controller->redirect($redirect);
	}

/**
 * A user has responded to a form and it has failed to save, redirect them to an error page and provide
 * a flash fail message.
 *
 * @param int   $formId       The id of the form submitted.
 * @param array $requestQuery The query data from the request.
 * @return void.
 */
	public function responseError($formId, $requestQuery) {
		$this->_controller->Flash->fail(
			[
				'title' => 'Your response failed to save.',
				'list' => $this->_controller->Form->FormResponse->validationErrors,
			]
		);

		if (!empty(Configure::read('EvFormBuilder.redirects.responseError'))) {
			$this->_controller->redirect(
				Configure::read('EvFormBuilder.redirects.responseError')
			);
		}
	}
}
