<?php

App::uses('EvFormBuilderAppModel', 'EvFormBuilder.Model');

class FormResponse extends EvFormBuilderAppModel {

	public $belongsTo = [
		'Form' => [
			'className' => 'EvFormBuilder.Form',
		],
		'User' => [
			'className' => 'EvCore.User',
		]
	];

	public $hasMany = [
		'QuestionBlockResponse' => [
			'className' => 'EvFormBuilder.QuestionBlockResponse',
		],
		'FormResponseAssociation' => [
			'className' => 'EvFormBuilder.FormResponseAssociation',
		],
	];

/**
 * Add the provided form associations to the responses.
 *
 * @param array $responses    The responses that have been submitted.
 * @param array $associations The associations in [model => model_id] format.
 * @return array The responses with associations added.
 */
	public function addFormResponseAssociations($responses, $associations) {
		if (empty($associations)) {
			return $responses;
		}

		//Check we can associate and get className
		foreach ($associations as $model => $modelId) {
			//If coming from the request query then they need to be camelised.
			$modelAlias = InflectorExt::camelize($model);

			if (empty(Configure::read('EvFormBuilder.associations.' . $modelAlias))) {
				unset($associations[$model]);
				continue;
			}

			$association = Configure::read('EvFormBuilder.associations.' . $modelAlias);

			foreach ($responses as $responseIndex => $response) {
				$responses[$responseIndex]['FormResponseAssociation'][] = [
					'model' => $association['className'],
					'model_id' => $modelId,
				];
			}
		}

		return $responses;
	}

/**
 * Format a form response so that it can be saved.
 *
 * @param array $response The responses that have been submitted.
 * @return array The formatted responses.
 */
	public function formatResponseData($response) {
		return $response;
	}
}
